#include "OscScreen.h"

BEGIN_EVENT_TABLE(OscScreen,wxGLCanvas)
    EVT_PAINT(OscScreen::OnPaint)
    EVT_SIZE(OscScreen::OnSize)
END_EVENT_TABLE()

OscScreen::OscScreen(wxWindow *parent, wxWindowID id, const wxPoint &pos, const wxSize &size)
          :wxGLCanvas(parent,id,pos,size)
{
    SetUpGL();
    
    Waveforms = NULL;
    
    CH = 0;
}

OscScreen::~OscScreen() {
}

void OscScreen::OnPaint(wxPaintEvent &event)
{
    wxPaintDC dc(this);
    SetCurrent();
    Render();
}

void OscScreen::Render()
{
    glClearColor(0.05f, 0.25f, 0.05f, 0.5f);
    glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);

    SetUpGL();
        
    try{
        RenderBackground();
        RenderSignal();
        DrawLabels();
    }catch(...) {}
    
    glFinish();
    SwapBuffers();
}

// this one draws the actual grid display
void OscScreen::SetUpGL()
{
    wxSize temp = this->GetSize();
    glViewport(0,0,temp.GetWidth(),temp.GetHeight());
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(-GRID_TRH,GRID_TRH,-GRID_TRH,GRID_TRH,0.0f,100.0f);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glClear(GL_DEPTH_BUFFER_BIT|GL_COLOR_BUFFER_BIT);
    glClearColor(0.05f,0.25f,0.05f,0.5f);
}

/*******
    How x and y are calculated?
    X - Scaled between grid start and end (512 x-points in the grid)
    Y - Waveforms[index] + ch_offset + GRID_TRAH
        - Waveforms[index] a sine value
        - ch_offset places wave to correct position in grid
        - GRID_TRAH places wave to middle (0 value) of grid
********/
void OscScreen::RenderSignal()
{
    if (!Waveforms)
        return;
        
    float ch_offset = -56.25f; //, A = 1.95f;
    GLdouble x;
    GLdouble y;

    float A = 1.0;

    ch_offset = 0;

    /*** 
    We draw a line according to sample data.
    ***/
    glColor3f(1.0f,1.0f,0.0f);
//    for (uint32_t i=CH; i < NO_CHANNELS; i++) {
      
    uint32_t i = CH;
      
        glBegin(GL_LINE_STRIP);
//        ch_offset += 6.25f;
        for (uint32_t j=0; j < CH_BUFFERSIZE; j++) {
            x = (GRID_AREA * ((double)j/(double)CH_BUFFERSIZE)) - GRID_HALF;
            y = (A*Waveforms[j+i*512]) + ch_offset ; //+ GRID_TRAH;
            glVertex2d(x,y);
        }
        glEnd();
//    }
}

void OscScreen::RenderBackground()
{
    int i;
    GLfloat x,y,x2,y2;

    for(i=0;i<=GRID_NO_OF_X;i++){
        x = (i * 100 / GRID_NO_OF_X) - GRID_HALF;
        y = -GRID_HALF; y2 = GRID_HALF;
        glBegin(GL_LINE_STRIP);
            glColor3f(0.2f,0.8f,0.2f);
            glVertex2f(x,y);
            glVertex2f(x,y2);
        glEnd();
    }
    for(i=0;i<=GRID_NO_OF_Y;i++){
        y = (i * 100 / GRID_NO_OF_Y) - GRID_HALF;
        x = -GRID_HALF; x2 = GRID_HALF;
        glBegin(GL_LINE_STRIP);
            glColor3f(0.2f,0.8f,0.2f);
            if (i == GRID_NO_OF_Y / 2) glColor3f(1,1,1);
            glVertex2f(x,y);
            glVertex2f(x2,y);
        glEnd();
    }
}

void OscScreen::SetWaveforms(float *buffer)
{
    if (buffer != Waveforms)
        Waveforms = buffer; // ptr xchg
}

void OscScreen::SetSize(wxSize size)
{
    SetUpGL();
}

void OscScreen::OnSize(wxSizeEvent &event)
{
    this->SetSize(event.GetSize());
}

// Reads pixel data from OpenGL canvas.
wxImage* OscScreen::readPixels(void)
{
     int32_t width, height, i, j;
     uint32_t nbBytes; // 1 byte = 8 bits = uint8_t = unsigned char
     
     this->GetClientSize(&width, &height);
     
     // Alloc pixel bytes
     nbBytes = 3 * width * height * sizeof(uint8_t);
     uint8_t *pTmpPixelData = new uint8_t[nbBytes];
     uint8_t *pPixelData = new uint8_t[nbBytes];
     
     // We should draw first the opengl image (prevents forming incorrect image)
     SetCurrent();
     Render();
     
     // Copy from OpenGL from point (0,0) to (width,height) mode RGB
     glReadPixels(0, 0, width, height, GL_RGB, GL_UNSIGNED_BYTE, pTmpPixelData);
     
     /* Swapping lines in the buffer.
     OpenGL writes from lower line going up but our image format works reverse (from up to down)*/
     for (j=0;j<height;j++) {// line counter
        for (i=0;i<width;i++) {// column counter
            pPixelData[(j*width+i)*3] = pTmpPixelData[((height-1-j)*width+i)*3];
            pPixelData[(j*width+i)*3+1] = pTmpPixelData[((height-1-j)*width+i)*3+1];
            pPixelData[(j*width+i)*3+2] = pTmpPixelData[((height-1-j)*width+i)*3+2];
        }
     }
     delete[] pTmpPixelData;
     
     return (new wxImage(width,height,pPixelData));
}

// Draws labels such as y(x) and volt/count markers to wxGL canvas
void OscScreen::DrawLabels(float voltage)
{
//    font.LoadFont(); // We have load font always on draw event.
    glColor3f(1.0f,1.0f,1.0f); // Set the color to white and display volts/counts

    //labels
/*    font.PrintText(0, 15,1,"%fV",voltage);
    font.PrintText(0, 95,1,"%fV",voltage);
    font.PrintText(0,175,1,"%fV",voltage);
    font.PrintText(0,255,1,"%fV",voltage);
    font.PrintText(0,305,1,"%fV",voltage);

	font.UnloadFont(); // Unload needs to be called so that next time font can be loaded succesfully
*/
}

// No description
float* OscScreen::GetWaveforms(void)
{
	return Waveforms;
}

void OscScreen::SetCH(int ch)
{
     CH = ch - 1;     
}
